// This program is free software: you can use, modify and/or redistribute it
// under the terms of the simplified BSD License. You should have received a
// copy of this license along this program. If not, see
// <http://www.opensource.org/licenses/bsd-license.html>.
//
// Copyright (C) 2017, Javier Sánchez Pérez <jsanchez@ulpgc.es>
// All rights reserved.

#include <stdio.h>
#include <malloc.h>

#include "histogram.h"
#include "utils.h"

#define max(x,y) (x>y)?x:y
#define HISTOGRAM_WIDTH 256

/*********************************************************************
 * NOTE:                                                             *
 * This file generates some information for the online demo in IPOL  *
 * It can be removed for using the program outside the demo system   *              
 *                                                                   *
 *********************************************************************/


void convert_histogram(
  unsigned char  *I,//output images
  float *histogram, //input histogram
  int    nx,        //number of columns
  int    ny,        //number of rows
  int    nz         //number of channels
)
{
  int N=HISTOGRAM_WIDTH+2;
  int M=202;
  int rate2=BIN_SIZE/HISTOGRAM_WIDTH;
  float rate=200*BIN_SIZE/(BIN_RATE*4.5*nx*ny);
  
  for(int i=0;i<M*N*nz;i++)
    I[i]=0;

  for(int i=1;i<N-1;i++)
    for(int c=0;c<nz;c++)
    {
      int v=0;
      for(int j=0;j<rate2;j++)
        v=max(v,(int)(rate*histogram[c*BIN_SIZE+(i-1)*rate2+j]+0.5));

      if(v>200) v=200;
      for(int j=2;j<=v;j++)
        I[(201-j)*N*nz+i*nz+c]=255;
    }
}


/**
  *
  *  Function to create an image of an accumulated color histogram
  *  and save to disk
  * 
**/
void convert_normalized_histogram(
  unsigned char  *I, //output image
  float *chistogram, //input histogram
  int    nz          //number of channels
)
{
  int N=HISTOGRAM_WIDTH+2;
  int M=202;
  int rate=BIN_SIZE/HISTOGRAM_WIDTH;
  for(int i=0;i<M*N*nz;i++)
    I[i]=0; 
  
  for(int i=1;i<N-2;i++)
    for(int c=0;c<nz;c++)
    {
      int v=(int) (200*chistogram[c*BIN_SIZE+(i-1)*rate]+0.5);
      for(int j=2;j<=v;j++)
	I[(201-j)*N*nz+i*nz+c]=255;
    }
}


void generate_histogram_view(
  char *video,
  char *h1,
  char *h2,
  int nx, 
  int ny, 
  int nz, 
  int nf
) 
{
  int N=HISTOGRAM_WIDTH+2;
  int M=202;
  int size=nx*ny*nz*nf;
  unsigned char *I1=new unsigned char[size];
  unsigned char *Ih=new unsigned char[M*N*nz*nf];
  float *chistograms=new float[BIN_SIZE*nz*nf];

  size_t r=read_video(video, I1, size);
 
  //create a video of accumulated histograms 
  #pragma omp parallel for
  for(int i=0;i<nf;i++)
    compute_channel_normalized_histogram(
      &I1[nx*ny*nz*i], &(chistograms[i*BIN_SIZE*nz]), nx, ny, nz
    );

  #pragma omp parallel for
  for(int i=0;i<nf;i++)
    convert_normalized_histogram(
      &Ih[M*N*nz*i], &chistograms[BIN_SIZE*nz*i], nz
    );
  
  write_video(h1, Ih, M*N*nz*nf);
  
  //create a video of histograms 
  #pragma omp parallel for
  for(int i=0;i<nf;i++)
    channel_histogram(
      &I1[nx*ny*nz*i], &(chistograms[i*BIN_SIZE*nz]),nx, ny, nz
    );

  #pragma omp parallel for
  for(int i=0;i<nf;i++)
    convert_histogram(
      &Ih[M*N*nz*i], &chistograms[BIN_SIZE*nz*i], nx, ny, nz
    );
  
  write_video(h2, Ih, M*N*nz*nf);
  
  delete []I1;
  delete []Ih;
  delete []chistograms;  
}



int main(int argc, char *argv[])
{
  if(argc<5)
    printf("\n\nUsage: %s input_raw_video output_raw_video out_dir "
           "nx ny nframes \n\n\n", argv[0]);
  else 
  {
    int nz=3;

    int i=1;
    char *input_raw_video=argv[i];  i++;
    char *output_raw_video=argv[i]; i++;
    char *out_dir=argv[i];  i++;
    int   nx=atoi(argv[i]); i++;
    int   ny=atoi(argv[i]); i++;
    int   nf=atoi(argv[i]); i++;

    char name1[500], name2[500];
    sprintf(name1,"%shistogram1.raw",out_dir);
    sprintf(name2,"%slhistogram1.raw",out_dir);
    generate_histogram_view(input_raw_video, name1, name2, nx, ny, nz, nf); 

    sprintf(name1,"%shistogram2.raw",out_dir);
    sprintf(name2,"%slhistogram2.raw",out_dir);
    generate_histogram_view(output_raw_video, name1, name2, nx, ny, nz, nf);
  } 
}


